@@ -14,8 +14,10 @@ import tempfile
 import urllib
 # TODO: may have to use urllib2.urlopen to get reasonable timeouts
 
+from ConfigParser import RawConfigParser
 from optparse import OptionParser
 
+
 # URL of -this file-
 location = 'http://k0s.org/mozilla/hg/MakeItSo/raw-file/tip/makeitso/makeitso.py'
 
@@ -178,7 +180,6 @@ class ContentTemplate(tempita.Template):
 
     def read_variables(self, variables):
         """read variables from stdin"""
-        # TODO: variables should (optionally) be richer objects
         retval = {}
         for i in variables:
             print 'Enter %s: ' % i,
@@ -320,23 +321,31 @@ class DirectoryTemplate(ContentTemplate):
 
 
 class PolyTemplate(ContentTemplate):
-    """template for several files/directories"""
+    """aggregate templates"""
 
     def __init__(self, templates, interactive=True, variables=None):
-
         self.interactive = interactive
         self.templates = []
+        entry_points = get_entry_points()
         for template in templates:
             if isinstance(template, basestring):
                 # TODO: check if the template is a [e.g] PasteScript.template entry point
                 if os.path.isdir(template):
                     self.templates.append(DirectoryTemplate(template, interactive=self.interactive, variables=variables))
+                elif not os.path.exists(template) and template in entry_points:
+                    self.templates.append(entry_points[template](interactive=interactive, variables=variables))
                 else:
                     self.templates.append(URITemplate(template, interactive=self.interactive, variables=variables))
             else:
                 # assume the template is an object that conforms to the API
                 self.templates.append(template)
 
+    def get_variables(self, **variables):
+        vars = variables.copy()
+        for template in self.templates:
+            vars.update(template.get_variables())
+        return vars
+
     def missing(self, **variables):
         vars = variables.copy()
         missing = set([])
@@ -362,7 +371,7 @@ class PolyTemplate(ContentTemplate):
         vars = self.get_variables(**variables)
         self.check_missing(vars)
 
-        # make the output directory
+        # make the output directory if multiple templates
         if output and len(self.templates) > 1 and not os.path.exists(output):
             os.makedirs(output)
 
@@ -372,6 +381,18 @@ class PolyTemplate(ContentTemplate):
         
 ### command line interface
 
+def read_config(config_files, templates=()):
+    """read variables from a set of .ini files"""
+    retval = {}
+    parser = RawConfigParser()
+    parser.read(config_files)
+    retval.update(dict(parser.items('DEFAULT')))
+    for template in templates:
+        if template in parser.sections():
+            retval.update(dict(parser.items(template)))
+    
+    return retval
+
 def invocation(url, **variables):
     """returns a string appropriate for TTW invocation"""
     variables_string = ' '.join(['%s=%s' % (i,j) for i,j in variables.items()])
@@ -383,6 +404,13 @@ def main(args=sys.argv[1:]):
     usage = '%prog [options] template <template> <...>'
     parser = OptionParser(usage, description=__doc__)
 
+    # find dotfile
+    dotfile = None
+    if 'HOME' in os.environ:
+        dotfile = os.path.join(os.environ['HOME'], '.makeitso')
+        if not (os.path.exists(dotfile) and os.path.isfile(dotfile)):
+            dotfile = None
+
     # delimeters
     # XXX needs tempita trunk
     if has_delimeters:
@@ -395,14 +423,16 @@ def main(args=sys.argv[1:]):
     parser.add_option('-o', '--output', dest='output',
                       help='where to put the output (filename or directory)')
 
-    # TODO
     # options for (.ini) variables
-#     parser.add_option('-c', '--config', dest='config'
-#                       default=[], action='append',
-#                       help='.ini config files to read variables from')
-#     parser.add_option('--no-defaults', dest='use_defaults',
-#                       default=True, action='store_false',
-#                       help="don't read ~/.makeitso") # XXX should only be displayed if ~/.makeitso exists
+    parser.add_option('-c', '--config', dest='config',
+                       default=[], action='append',
+                       help='.ini config files to read variables from')
+    if dotfile:
+        parser.add_option('--no-defaults', dest='use_defaults',
+                          default=True, action='store_false',
+                          help="don't read ~/.makeitso")
+
+    # TODO
 #     parser.add_option('-u', '--update', dest='update',
 #                       help="update the specified .ini file for variables read from this run")
 #     parser.add_option('-U', '--Update', dest='Update',
@@ -418,9 +448,25 @@ def main(args=sys.argv[1:]):
     parser.add_option('--variables', dest='variables',
                       action='store_true', default=False,
                       help='print the variables in a template')
+    entry_points = get_entry_points()
+    if entry_points:
+        parser.add_option('--list-templates', dest='list_templates',
+                          action='store_true', default=False,
+                          help="list installed MakeItSo! templates")
+
     
     options, args = parser.parse_args(args)
 
+    # list the templates
+    if entry_points and options.list_templates:
+        for key in sorted(entry_points.keys()):
+            template_class = entry_points[key]
+            description = getattr(template_class, 'description', '')
+            description = description or getattr(template_class, '__doc__', '')
+            description = description.strip()
+            print key + ': ' + description
+        return
+
     # print the variables for the templates
     if options.variables:
 
@@ -441,13 +487,24 @@ def main(args=sys.argv[1:]):
     # template variables
     variables = {}
     _args = []
+
+    # read variables from configuration
+    config_files = options.config
+    if dotfile and options.use_defaults:
+        config_files.insert(0, dotfile)
+    if config_files:
+        variables.update(read_config(config_files))
+
+    # override with variables from the command line
+    _variables = {}
     for arg in args:
         if '=' in arg:
             key, value = arg.split('=')
-            variables[key] = value
+            _variables[key] = value
         else:
             _args.append(arg)
     args = _args
+    variables.update(_variables)
 
     # print TTW commandline for invocation
     if options.commandline:
@@ -458,11 +515,12 @@ def main(args=sys.argv[1:]):
             print invocation('[URI]', **variables)
         return
 
+
     # get the content
     if args:
         template = PolyTemplate(templates=args,
                                 variables=variables)
-        template.substitute(output=options.output)
+        template.substitute({}, output=options.output)
     else:
         template = ContentTemplate(sys.stdin.read(), variables=variables)
         print template.substitute()
@@ -472,3 +530,21 @@ def main(args=sys.argv[1:]):
         
 if __name__ == '__main__':
     main()
+
+
+# get templates from pkg_resources
+# (MakeItSo! and [TODO] pastescript templates)
+# this should go last to ensure the module is wholly loaded [?]
+def get_entry_points():
+    retval = {}
+    try:
+        from pkg_resources import iter_entry_points
+        for i in iter_entry_points('makeitso.templates'):
+            try:
+                retval[i.name] = i.load()
+            except:
+                continue
+
+    except ImportError:
+        return retval
+    return retval
